/**
 * match module import
 *
 * eg:
 * import { some } from './module';
 * const some = await import('./module')
 */
const reg = /import(?:(?:[\w*{},\s]+from\s+)|(?:\())['"](.*)['"]/g;
const defaultExtension = (file) => {
    if (/\/$/.test(file)) {
        return `${file}index.js`;
    }
    if (!/\.(\w+)$/.test(file)) {
        return `${file}.js`;
    }
    return file;
};
/**
 * 解析script中的es module导入路径
 * @author kianfang
 * @date 2023-11-14
 *
 * @param {string} script
 * @param {URL} baseUrl
 * @returns {string}
 */
function resolveModulePath (script, option) {
    const { base, polyfill = true } = option;
    const isSpecialPath = (path) => ['./', '../', '/'].some((prefix) => path.startsWith(prefix));
    const result = script.replace(reg, (match, path) => {
        if (isSpecialPath(path)) {
            const url = new URL(path, base);
            return match.replace(path, defaultExtension(url.href));
        }
        return match;
    });
    if (polyfill) {
        // polyfill import.meta.url and import.meta.resolve
        const meta = 'import.meta';
        const prefix = [
            `${meta}.url='${base.href}';`,
            `${meta}._resolve=${meta}.resolve;`,
            `${meta}.resolve=function(m){return (${isSpecialPath.toString()})(m)?(${defaultExtension.toString()})(new URL(m, this.url).href):this._resolve(m)}`,
        ].join('\n');
        return `${prefix}${result}`;
    }
    return result;
}

/**
 * Vue浏览器端SFC加载器
 * @author kianfang
 * @date 2023-05-10
 *
 * @see {@link https://github.com/kianfang/vue-import}
 *
 * @param {string} url
 * @param {ComponentOptions} props
 * @returns {ComponentOptions}
 */
async function index (url, props = {}) {
    const res = await fetch(url);
    const sfc = await res.text();
    const ele = document.createElement('html');
    ele.innerHTML = sfc;
    const template = ele.querySelector('template')?.innerHTML;
    const styleList = ele.querySelectorAll('style');
    const script = ele.querySelector('script')?.innerHTML;
    let sfcProps = {};
    // 1、es module无法从外部js直接获取导出模块
    // 2、import()只支持url导入，这里使用blob url解决
    if (script) {
        const resolvedScript = resolveModulePath(script, { base: new URL(res.url) });
        const blob = new Blob([resolvedScript], { type: 'application/javascript' });
        const blobUrl = URL.createObjectURL(blob);
        const importModule = await import(blobUrl);
        sfcProps = importModule.default;
    }
    const { beforeMount, beforeUnmount, ...loaderProps } = props;
    const component = {
        template,
        ...sfcProps,
        beforeMount() {
            styleList.forEach((style) => {
                document.head.appendChild(style);
            });
            // beforeMount方法重写，这里手动调用
            sfcProps.beforeMount?.call(this);
            beforeMount?.call(this);
        },
        beforeUnmount() {
            styleList.forEach((style) => style.remove());
            // beforeUnmount方法重写，这里手动调用
            sfcProps.beforeUnmount?.call(this);
            beforeUnmount?.call(this);
        },
        ...loaderProps,
    };
    return component;
}

export { index as default };
//# sourceMappingURL=vue-import.esm-browser.js.map
